@@ -48,7 +48,6 @@
 
 #define KE_TIME_WAIT 			5000
 #define COUNT_OF_ITEM_IN_LAYER_KEYSET 	16
-#define COUNT_OF_ITEM_IN_KEYECHO_KEYSET 16
 
 #define MODIFIER(K,C) (K ? C : "")
 #define LINE(X)       (X != 0 ? "-" : "")
@@ -196,14 +195,6 @@ static GSList 			   *accessible_key_set_list   = NULL;
 static GSList 			   *ke_user_key_list 	 = NULL;
 static GSList 			   *reg_list = NULL;
 
-/**
- * Workaround for bug #129951:
- * Some at-spi key press events are received for four times, 
-   with the same timestamp. A single event with a particular 
-   timestamp value will be fired, the other three will be rejected. 
- */
-static glong timestamp;
-
 /**
  * keyecho
 **/
@@ -408,10 +399,11 @@ ke_return_keystring (gint keyID)
         case XK_Down:	rv = g_strdup("Down");break;
         case XK_Page_Up:rv = g_strdup("Page_Up");break;
         case XK_Page_Down:rv = g_strdup("Page_Down");break;
+	case XK_BackSpace:rv = g_strdup ("BackSpace");break;
+	case XK_Delete:rv = g_strdup ("Delete");break;
         default:
     	    rv = NULL;
     }
-    
     sru_debug ("Return keystring:%s",rv);
     
     return rv;
@@ -500,10 +492,6 @@ ke_report_keyecho_event (const AccessibleKeystroke *key, gpointer user_data)
 	     (long int) key->timestamp,
 	     key->keystring);  
 	     
-    if ((glong) key->timestamp == timestamp)
-	return FALSE;
-    timestamp = (glong) key->timestamp;		      
-
     ke_log_at_spi (key);    
     
     if (key->type == SPI_KEY_RELEASED)
@@ -1254,24 +1242,6 @@ ke_register_keysetitem (KeySetItem *ksi)
 				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
 				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
     ke_print_register_return_value (retval, "");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
-				    			(AccessibleKeySet *) ksi->keyset,
-				    			ksi->modifier | SPI_KEYMASK_BUTTON1,
-				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
-				    			(AccessibleKeySet *) ksi->keyset,
-				    			ksi->modifier | SPI_KEYMASK_BUTTON2,
-				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_user_key_listener,
-				    			(AccessibleKeySet *) ksi->keyset,
-				    			ksi->modifier | SPI_KEYMASK_BUTTON3,
-				    			(gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				    			SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "");
 
     return TRUE;
 }
@@ -1361,60 +1331,6 @@ ke_layer_register_events (AccessibleKeystrokeListener *ke_layer_listener,
 	Layer key listener. Listen key event from numpad. Listen only if a numlock 
 	is active.
     */
-#ifndef USE_ALL_WINDOWS
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON1 | SPI_KEYMASK_NUMLOCK,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "(B1|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON2 | SPI_KEYMASK_NUMLOCK,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "(B2|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON3 | SPI_KEYMASK_NUMLOCK,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-			    	      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "(B3|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON1 | 
-				      SPI_KEYMASK_BUTTON2 |
-				      SPI_KEYMASK_NUMLOCK,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "(B1|B2|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON1 | 
-				      SPI_KEYMASK_BUTTON3 |
-				      SPI_KEYMASK_NUMLOCK,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "(B1|B3|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON3 |
-				      SPI_KEYMASK_BUTTON2 |
-				      SPI_KEYMASK_NUMLOCK,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "(B2|B3|NUMLOCK)");
-    retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
-				      (AccessibleKeySet *) ke_layer_keyset,
-				      SPI_KEYMASK_BUTTON3 |
-				      SPI_KEYMASK_BUTTON2 |
-				      SPI_KEYMASK_BUTTON1 |
-				      SPI_KEYMASK_NUMLOCK,
-				      (gulong) ( SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    ke_print_register_return_value (retval, "(B1|B2|B3|NUMLOCK)");
-#endif
-
     retval = SPI_registerAccessibleKeystrokeListener (ke_layer_listener,
 				      (AccessibleKeySet *) ke_layer_keyset,
 				      SPI_KEYMASK_NUMLOCK,
@@ -1511,22 +1427,6 @@ ke_keyecho_register_events (AccessibleKeystrokeListener *ke_key_listener,
 	SHIFT_L, SHIFT_R, Control_L, Control_R, Up, Down, Left, Right, 
 	PageUp, PageDown, Home, End, CapsLock, NumLock,
     */
-    retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
-				      (AccessibleKeySet *) ke_key_keyset,
-				      SPI_KEYMASK_BUTTON1,
-				      (gulong) (SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
-				      (AccessibleKeySet *) ke_key_keyset,
-				      SPI_KEYMASK_BUTTON2,
-				      (gulong) (SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-    retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
-				      (AccessibleKeySet *) ke_key_keyset,
-				      SPI_KEYMASK_BUTTON3,
-				      (gulong) (SPI_KEY_PRESSED | SPI_KEY_RELEASED ),
-				      SPI_KEYLISTENER_NOSYNC | SPI_KEYLISTENER_CANCONSUME);
-				      
     retval = SPI_registerAccessibleKeystrokeListener (ke_key_listener,
 				      (AccessibleKeySet *) ke_key_keyset,
 				      SPI_KEYMASK_UNMODIFIED,
@@ -1601,7 +1501,7 @@ ke_get_keypad_keycodes (void)
  * return: keysym array
  *
 **/
-long keyecho_keysym[COUNT_OF_ITEM_IN_KEYECHO_KEYSET] = { 
+long keyecho_keysym[] = { 
     XK_Alt_L, 
     XK_Alt_R,
     XK_Shift_L,
@@ -1617,7 +1517,9 @@ long keyecho_keysym[COUNT_OF_ITEM_IN_KEYECHO_KEYSET] = {
     XK_Up,
     XK_Down,
     XK_Page_Up,
-    XK_Page_Down
+    XK_Page_Down,
+    XK_BackSpace,
+    XK_Delete
 };
 
 gchar*
@@ -1625,10 +1527,14 @@ ke_get_keyecho_keysyms (void)
 {
     gchar *rv = NULL;
     gint i;
+    Display *disp;
+    disp = GDK_DISPLAY ();
 
     for (i = 0 ; i < G_N_ELEMENTS(keyecho_keysym) ; i++)
     {
-        if (g_unichar_validate ((guint32)keyecho_keysym[i]))
+	short key;
+	key = XKeysymToKeycode (disp, keyecho_keysym[i]);
+	if (g_unichar_validate ((guint32)key)) 
         {
 	    gchar *keysym = NULL;
     	    gchar *tmp = NULL;
@@ -1784,7 +1690,7 @@ ke_init (KeyboardEchoCB kecb)
 	Keyboard echo.
     */
     ke_keyecho_keysyms = ke_get_keyecho_keysyms ();
-    ke_keyecho_keyset  = SPI_createAccessibleKeySet (COUNT_OF_ITEM_IN_KEYECHO_KEYSET, ke_keyecho_keysyms, NULL, NULL);
+    ke_keyecho_keyset  = SPI_createAccessibleKeySet (G_N_ELEMENTS (keyecho_keysym),ke_keyecho_keysyms, NULL, NULL);
     g_free (ke_keyecho_keysyms);
 
     /*
@@ -1835,18 +1741,6 @@ ke_user_key_list_unregister_events (void)
 	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
 						   ksi->modifier);
 
-	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
-						   ksi->modifier | 
-						   SPI_KEYMASK_BUTTON1);
-
-	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
-						   ksi->modifier |
-						   SPI_KEYMASK_BUTTON2);
-
-	SPI_deregisterAccessibleKeystrokeListener (ke_user_key_listener, 
-						   ksi->modifier |
-						   SPI_KEYMASK_BUTTON3);
-	
 	ke_key_set_item_free (ksi);
 	
 	elem->data = NULL;
@@ -1864,35 +1758,6 @@ ke_user_key_list_unregister_events (void)
 static void 
 ke_layer_unregister_events (void)
 {
-#ifndef USE_ALL_WINDOWS
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_BUTTON1 |
-					    SPI_KEYMASK_NUMLOCK);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_BUTTON2 |
-					    SPI_KEYMASK_NUMLOCK);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_BUTTON3 |
-					    SPI_KEYMASK_NUMLOCK);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_BUTTON1 |
-					    SPI_KEYMASK_BUTTON2 |
-					    SPI_KEYMASK_NUMLOCK);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_BUTTON1 |
-					    SPI_KEYMASK_BUTTON3 |
-					    SPI_KEYMASK_NUMLOCK);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_BUTTON2 |
-					    SPI_KEYMASK_BUTTON3 |
-					    SPI_KEYMASK_NUMLOCK);
-    SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
-					    SPI_KEYMASK_BUTTON1 |
-					    SPI_KEYMASK_BUTTON2 |
-					    SPI_KEYMASK_BUTTON3 |
-					    SPI_KEYMASK_NUMLOCK);
-#endif
-
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
 					    SPI_KEYMASK_NUMLOCK);
     SPI_deregisterAccessibleKeystrokeListener (ke_layer_listener, 
@@ -1925,12 +1790,6 @@ static void
 ke_keyecho_unregister_events (void)
 {
     /* deregister keylisteners */
-    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
-					    SPI_KEYMASK_BUTTON1);
-    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
-					    SPI_KEYMASK_BUTTON2);
-    SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
-					    SPI_KEYMASK_BUTTON3);
     SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
 					    SPI_KEYMASK_UNMODIFIED );
     SPI_deregisterAccessibleKeystrokeListener (ke_keyecho_listener, 
