@@ -2121,7 +2121,7 @@ src_process_cursor (gchar *cursor)
     {
 	if (strcmp (cursor, "Home") == 0)
 	    src_last_cursor_echo = g_strdup (_("home"));
-	else if (strcmp (cursor, "End") == 0)
+        else if (strcmp (cursor, "End") == 0)
     	    src_last_cursor_echo = g_strdup (_("end"));
 	else if (strcmp (cursor, "Page_Up") == 0)
 	    src_last_cursor_echo = g_strdup (_("page up"));
@@ -2135,6 +2135,10 @@ src_process_cursor (gchar *cursor)
 	    src_last_cursor_echo = g_strdup (_("down"));
 	else if (strcmp (cursor, "Up") == 0)
 	    src_last_cursor_echo = g_strdup (_("up"));
+	else if (strcmp (cursor, "BackSpace") == 0)
+	    src_last_cursor_echo = g_strdup (_("backspace"));
+	else if (strcmp (cursor, "Delete") == 0)
+	    src_last_cursor_echo = g_strdup (_("delete"));
 	else
 	    sru_assert_not_reached ();
 	        
@@ -2185,7 +2189,9 @@ src_process_key (SREvent *event,
 	     strcmp (srhotkey_data->keystring, "Right") == 0 ||
 	     strcmp (srhotkey_data->keystring, "Left") == 0 ||
 	     strcmp (srhotkey_data->keystring, "Down") == 0 ||
-	     strcmp (srhotkey_data->keystring, "Up") == 0)
+	     strcmp (srhotkey_data->keystring, "Up") == 0 ||
+	     strcmp (srhotkey_data->keystring, "BackSpace") == 0 ||
+	     strcmp (srhotkey_data->keystring, "Delete") == 0)
     {
 	src_process_cursor (srhotkey_data->keystring);
     }
